/**
 * FetchItems is a logic component that renders no content itself. Its job
 * is to manage when the given fetchMore method should be called based on
 * the given scroll/size props.
 *
 * While no element is actually passed to FetchItems, it is intended to be used
 * in conjunction with a large scroll container that uses async fetching to
 * load and render additional data. Based on the height of this container and
 * its current scroll position, FetchItems is responsible for triggering future
 * fetch calls.
 */
import * as React from 'react';
import PropTypes from 'prop-types';
interface Props {
    containerHeight: number;
    isAtEnd?: boolean;
    isFetching: boolean;
    fetchMore?: () => void;
    scrollHeight: number;
    scrollTop: number;
}
export default class FetchItems extends React.PureComponent<Props> {
    static propTypes: {
        containerHeight: PropTypes.Validator<number>;
        isAtEnd: PropTypes.Requireable<boolean>;
        isFetching: PropTypes.Validator<boolean>;
        fetchMore: PropTypes.Requireable<(...args: any[]) => any>;
        scrollHeight: PropTypes.Validator<number>;
        scrollTop: PropTypes.Validator<number>;
    };
    componentDidMount(): void;
    componentDidUpdate(): void;
    check: () => void;
    render(): null;
}
export {};
