/**
 * FetchItems is a logic component that renders no content itself. Its job
 * is to manage when the given fetchMore method should be called based on
 * the given scroll/size props.
 *
 * While no element is actually passed to FetchItems, it is intended to be used
 * in conjunction with a large scroll container that uses async fetching to
 * load and render additional data. Based on the height of this container and
 * its current scroll position, FetchItems is responsible for triggering future
 * fetch calls.
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as React from 'react';
import PropTypes from 'prop-types';
var FetchItems = /** @class */ (function (_super) {
    __extends(FetchItems, _super);
    function FetchItems() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.check = function () {
            var _a = _this.props, containerHeight = _a.containerHeight, isAtEnd = _a.isAtEnd, isFetching = _a.isFetching, fetchMore = _a.fetchMore, scrollHeight = _a.scrollHeight, scrollTop = _a.scrollTop;
            if (isAtEnd || isFetching || !fetchMore) {
                return;
            }
            var scrollBuffer = containerHeight * 3;
            if (scrollTop + scrollBuffer > scrollHeight) {
                fetchMore();
            }
        };
        return _this;
    }
    FetchItems.prototype.componentDidMount = function () {
        setTimeout(this.check);
    };
    FetchItems.prototype.componentDidUpdate = function () {
        this.check();
    };
    FetchItems.prototype.render = function () {
        return null;
    };
    FetchItems.propTypes = {
        containerHeight: PropTypes.number.isRequired,
        isAtEnd: PropTypes.bool,
        isFetching: PropTypes.bool.isRequired,
        fetchMore: PropTypes.func,
        scrollHeight: PropTypes.number.isRequired,
        scrollTop: PropTypes.number.isRequired,
    };
    return FetchItems;
}(React.PureComponent));
export default FetchItems;
//# sourceMappingURL=FetchItems.js.map