var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as React from 'react';
import PropTypes from 'prop-types';
import debounce from './debounce.js';
import FetchItems from './FetchItems.js';
import ScrollContainer from './ScrollContainer.js';
import throttle from './throttle.js';
import MeasurementStore from './MeasurementStore.js';
import { getElementHeight, getRelativeScrollTop, getScrollPos, } from './scrollUtils.js';
import { DefaultLayoutSymbol, UniformRowLayoutSymbol, } from './legacyLayoutSymbols.js';
import defaultLayout from './defaultLayout.js';
import uniformRowLayout from './uniformRowLayout.js';
import fullWidthLayout from './fullWidthLayout.js';
import LegacyMasonryLayout from './layouts/MasonryLayout.js';
import LegacyUniformRowLayout from './layouts/UniformRowLayout.js';
;
var RESIZE_DEBOUNCE = 300;
// Multiplied against container height.
// The amount of extra buffer space for populating visible items.
var VIRTUAL_BUFFER_FACTOR = 0.7;
var layoutNumberToCssDimension = function (n) { return (n !== Infinity ? n : undefined); };
var Masonry = /** @class */ (function (_super) {
    __extends(Masonry, _super);
    function Masonry(props) {
        var _this = _super.call(this, props) || this;
        /**
         * Delays resize handling in case the scroll container is still being resized.
         */
        _this.handleResize = debounce(function () {
            if (_this.gridWrapper) {
                _this.setState({ width: _this.gridWrapper.clientWidth });
            }
        }, RESIZE_DEBOUNCE);
        _this.updateScrollPosition = throttle(function () {
            if (!_this.scrollContainer) {
                return;
            }
            var scrollContainer = _this.scrollContainer.getScrollContainerRef();
            if (!scrollContainer) {
                return;
            }
            _this.setState({
                scrollTop: getScrollPos(scrollContainer),
            });
        });
        _this.measureContainerAsync = debounce(function () {
            _this.measureContainer();
        }, 0);
        _this.setGridWrapperRef = function (ref) {
            _this.gridWrapper = ref;
        };
        _this.setScrollContainerRef = function (ref) {
            _this.scrollContainer = ref;
        };
        _this.fetchMore = function () {
            var loadItems = _this.props.loadItems;
            if (loadItems && typeof loadItems === 'function') {
                _this.setState({
                    isFetching: true,
                }, function () { return loadItems({ from: _this.props.items.length }); });
            }
        };
        _this.gridWrapper = null;
        _this.scrollContainer = null;
        _this.renderMasonryComponent = function (itemData, idx, position) {
            var _a = _this.props, Component = _a.comp, virtualize = _a.virtualize, virtualBoundsTop = _a.virtualBoundsTop, virtualBoundsBottom = _a.virtualBoundsBottom, itemClassName = _a.itemClassName;
            var top = position.top, left = position.left, width = position.width, height = position.height;
            var isVisible;
            if (_this.props.scrollContainer) {
                var virtualBuffer = _this.containerHeight * VIRTUAL_BUFFER_FACTOR;
                var offsetScrollPos = _this.state.scrollTop - _this.containerOffset;
                var viewportTop = virtualBoundsTop
                    ? offsetScrollPos - virtualBoundsTop
                    : offsetScrollPos - virtualBuffer;
                var viewportBottom = virtualBoundsBottom
                    ? offsetScrollPos + _this.containerHeight + virtualBoundsBottom
                    : offsetScrollPos + _this.containerHeight + virtualBuffer;
                isVisible = !(position.top + position.height < viewportTop ||
                    position.top > viewportBottom);
            }
            else {
                // if no scroll container is passed in, items should always be visible
                isVisible = true;
            }
            var itemComponent = (React.createElement("div", { key: "item-" + idx, className: itemClassName, "data-grid-item": true, style: {
                    position: 'absolute',
                    top: 0,
                    left: 0,
                    transform: "translateX(" + left + "px) translateY(" + top + "px)",
                    WebkitTransform: "translateX(" + left + "px) translateY(" + top + "px)",
                    width: layoutNumberToCssDimension(width),
                    height: layoutNumberToCssDimension(height),
                } },
                React.createElement(Component, { data: itemData, itemIdx: idx, isMeasuring: false })));
            return virtualize ? (isVisible && itemComponent) || null : itemComponent;
        };
        _this.containerHeight = 0;
        _this.containerOffset = 0;
        _this.state = {
            hasPendingMeasurements: props.items.some(function (item) { return !!item && !props.measurementStore.has(item); }),
            isFetching: false,
            // eslint-disable-next-line react/no-unused-state
            items: props.items,
            scrollTop: 0,
            width: undefined,
        };
        return _this;
    }
    Masonry.createMeasurementStore = function () {
        return new MeasurementStore();
    };
    /**
     * Adds hooks after the component mounts.
     */
    Masonry.prototype.componentDidMount = function () {
        var _this = this;
        window.addEventListener('resize', this.handleResize);
        this.measureContainer();
        var scrollTop = this.state.scrollTop;
        if (this.scrollContainer != null) {
            var scrollContainer = this.scrollContainer.getScrollContainerRef();
            if (scrollContainer) {
                scrollTop = getScrollPos(scrollContainer);
            }
        }
        this.setState(function (prevState) { return ({
            scrollTop: scrollTop,
            width: _this.gridWrapper ? _this.gridWrapper.clientWidth : prevState.width,
        }); });
    };
    Masonry.prototype.componentDidUpdate = function (_prevProps, prevState) {
        var _this = this;
        var _a = this.props, items = _a.items, measurementStore = _a.measurementStore;
        this.measureContainerAsync();
        if (prevState.width != null && this.state.width !== prevState.width) {
            measurementStore.reset();
        }
        // calculate whether we still have pending measurements
        var hasPendingMeasurements = items.some(function (item) { return !!item && !measurementStore.has(item); });
        if (hasPendingMeasurements ||
            hasPendingMeasurements !== this.state.hasPendingMeasurements ||
            prevState.width == null) {
            this.insertAnimationFrame = requestAnimationFrame(function () {
                _this.setState({
                    hasPendingMeasurements: hasPendingMeasurements,
                });
            });
        }
    };
    /**
     * Remove listeners when unmounting.
     */
    Masonry.prototype.componentWillUnmount = function () {
        if (this.insertAnimationFrame) {
            cancelAnimationFrame(this.insertAnimationFrame);
        }
        // Make sure async methods are cancelled.
        this.measureContainerAsync.clearTimeout();
        this.handleResize.clearTimeout();
        this.updateScrollPosition.clearTimeout();
        window.removeEventListener('resize', this.handleResize);
    };
    Masonry.getDerivedStateFromProps = function (props, state) {
        var items = props.items, measurementStore = props.measurementStore;
        // whenever we're receiving new props, determine whether any items need to be measured
        // TODO - we should treat items as immutable
        var hasPendingMeasurements = items.some(function (item) { return !measurementStore.has(item); });
        // Shallow compare all items, if any change reflow the grid.
        for (var i = 0; i < items.length; i += 1) {
            // We've reached the end of our current props and everything matches.
            // If we hit this case it means we need to insert new items.
            if (state.items[i] === undefined) {
                return {
                    hasPendingMeasurements: hasPendingMeasurements,
                    items: items,
                    isFetching: false,
                };
            }
            // Reset grid items when:
            if (
            // An item object ref does not match.
            items[i] !== state.items[i] ||
                // Or less items than we currently have are passed in.
                items.length < state.items.length) {
                return {
                    hasPendingMeasurements: hasPendingMeasurements,
                    items: items,
                    isFetching: false,
                };
            }
        }
        // Reset items if new items array is empty.
        if (items.length === 0 && state.items.length > 0) {
            return {
                hasPendingMeasurements: hasPendingMeasurements,
                items: items,
                isFetching: false,
            };
        }
        if (hasPendingMeasurements !== state.hasPendingMeasurements) {
            // make sure we always update hasPendingMeasurements
            return {
                hasPendingMeasurements: hasPendingMeasurements,
                items: items,
            };
        }
        // Return null to indicate no change to state.
        return null;
    };
    Masonry.prototype.measureContainer = function () {
        if (this.scrollContainer != null) {
            var scrollContainer = this.scrollContainer;
            var scrollContainerRef = scrollContainer.getScrollContainerRef();
            if (scrollContainerRef) {
                this.containerHeight = getElementHeight(scrollContainerRef);
                var el = this.gridWrapper;
                if (el instanceof HTMLElement) {
                    var relativeScrollTop = getRelativeScrollTop(scrollContainerRef);
                    this.containerOffset =
                        el.getBoundingClientRect().top + relativeScrollTop;
                }
            }
        }
    };
    /**
     * Clear measurements/positions and force a reflow of the entire grid.
     * Only use this if absolutely necessary - ex: We need to reflow items if the
     * number of columns we would display should change after a resize.
     */
    Masonry.prototype.reflow = function () {
        this.props.measurementStore.reset();
        this.measureContainer();
        this.forceUpdate();
    };
    Masonry.prototype.render = function () {
        var _this = this;
        var _a = this.props, columnWidth = _a.columnWidth, Component = _a.comp, flexible = _a.flexible, gutter = _a.gutterWidth, measurementStore = _a.measurementStore, items = _a.items, minCols = _a.minCols, className = _a.className;
        var _b = this.state, hasPendingMeasurements = _b.hasPendingMeasurements, width = _b.width;
        var layout, dataMasonry;
        if (flexible && width !== null) {
            dataMasonry = {
                'data-masonry-layout': 'fullWidth',
                'data-masonry-ideal-column-width': columnWidth || 240,
                'data-masonry-gutter': gutter || 0,
                'data-masonry-min-cols': minCols || 2,
            };
            layout = fullWidthLayout({
                gutter: gutter,
                cache: measurementStore,
                minCols: minCols,
                idealColumnWidth: columnWidth,
                width: width,
            });
        }
        else if (this.props.layout === UniformRowLayoutSymbol ||
            this.props.layout instanceof LegacyUniformRowLayout) {
            dataMasonry = {
                'data-masonry-layout': 'uniformRow',
                'data-masonry-column-width': columnWidth || 236,
                'data-masonry-gutter': gutter || 14,
                'data-masonry-min-cols': minCols || 3,
            };
            layout = uniformRowLayout({
                cache: measurementStore,
                columnWidth: columnWidth,
                gutter: gutter,
                minCols: minCols,
                width: width,
            });
        }
        else {
            dataMasonry = {
                'data-masonry-layout': 'default',
                'data-masonry-column-width': columnWidth || 236,
                'data-masonry-gutter': gutter || 14,
                'data-masonry-min-cols': minCols || 2,
            };
            layout = defaultLayout({
                cache: measurementStore,
                columnWidth: columnWidth,
                gutter: gutter,
                minCols: minCols,
                width: width,
            });
        }
        var gridBody;
        if (width == null && hasPendingMeasurements) {
            // When hyrdating from a server render, we don't have the width of the grid
            // and the measurement store is empty
            gridBody = (React.createElement("div", __assign({ className: className, style: {
                    position: 'relative',
                    margin: '0 auto',
                    height: 0,
                    width: '100%',
                }, ref: this.setGridWrapperRef }, dataMasonry), items.filter(function (item) { return item; }).map(function (item, i) { return (React.createElement("div", { className: "static", "data-grid-item": true, key: i, style: {
                    visibility: 'hidden',
                    position: 'absolute',
                    top: 0,
                    left: 0,
                    transform: 'translateX(0px) translateY(0px)',
                    WebkitTransform: 'translateX(0px) translateY(0px)',
                    width: flexible
                        ? undefined
                        : layoutNumberToCssDimension(columnWidth || Infinity),
                }, ref: function (el) {
                    if (el && !flexible) {
                        // only measure flexible items on client
                        measurementStore.set(item, el.clientHeight);
                    }
                } },
                React.createElement(Component, { data: item, itemIdx: i, isMeasuring: false }))); })));
        }
        else if (width == null) {
            // When the width is empty (usually after a re-mount) render an empty
            // div to collect the width for layout
            gridBody = React.createElement("div", { style: { width: '100%' }, ref: this.setGridWrapperRef });
        }
        else {
            // Full layout is possible
            var itemsToRender_1 = items.filter(function (item) { return item && measurementStore.has(item); });
            var itemsToMeasure = items
                .filter(function (item) { return item && !measurementStore.has(item); })
                .slice(0, minCols);
            var positions_1 = layout(itemsToRender_1);
            var measuringPositions_1 = layout(itemsToMeasure);
            // Math.max() === -Infinity when there are no positions
            var height = positions_1.length
                ? Math.max.apply(Math, positions_1.map(function (pos) { return pos.top + pos.height; })) : 0;
            gridBody = (React.createElement("div", { style: { width: '100%' }, ref: this.setGridWrapperRef },
                React.createElement("div", { className: className, style: {
                        position: 'relative',
                        margin: '0 auto',
                        height: height,
                        width: width,
                    } }, itemsToRender_1.map(function (item, i) {
                    return _this.renderMasonryComponent(item, i, positions_1[i]);
                })),
                React.createElement("div", { className: className, style: {
                        position: 'relative',
                        margin: '0 auto',
                        height: '100%',
                        width: width,
                    } }, itemsToMeasure.map(function (data, i) {
                    // itemsToMeasure is always the length of minCols, so i will always be 0..minCols.length
                    // we normalize the index here relative to the item list as a whole so that itemIdx is correct
                    // and so that React doesnt reuse the measurement nodes
                    var measurementIndex = itemsToRender_1.length + i;
                    var position = measuringPositions_1[i];
                    return (React.createElement("div", { key: "measuring-" + measurementIndex, style: {
                            visibility: 'hidden',
                            position: 'absolute',
                            top: layoutNumberToCssDimension(position.top),
                            left: layoutNumberToCssDimension(position.left),
                            width: layoutNumberToCssDimension(position.width),
                            height: layoutNumberToCssDimension(position.height),
                        }, ref: function (el) {
                            if (el) {
                                measurementStore.set(data, el.clientHeight);
                            }
                        } },
                        React.createElement(Component, { data: data, itemIdx: measurementIndex, isMeasuring: true })));
                })),
                this.scrollContainer && (React.createElement(FetchItems, { containerHeight: this.containerHeight, fetchMore: this.fetchMore, isFetching: this.state.isFetching || this.state.hasPendingMeasurements, scrollHeight: height, scrollTop: this.state.scrollTop }))));
        }
        return this.props.scrollContainer ? (React.createElement(ScrollContainer, { ref: this.setScrollContainerRef, onScroll: this.updateScrollPosition, scrollContainer: this.props.scrollContainer }, gridBody)) : (gridBody);
    };
    Masonry.propTypes = {
        /**
         * The preferred/target item width. If `flexible` is set, the item width will
         * grow to fill column space, and shrink to fit if below min columns.
         */
        columnWidth: PropTypes.number,
        /**
         * The component to render.
         */
        /* eslint react/no-unused-prop-types: 0 */
        comp: PropTypes.func.isRequired,
        /**
         * The preferred/target item width. Item width will grow to fill
         * column space, and shrink to fit if below min columns.
         */
        flexible: PropTypes.bool,
        /**
         * The amount of space between each item.
         */
        gutterWidth: PropTypes.number,
        /**
         * An array of all objects to display in the grid.
         */
        items: PropTypes.arrayOf(PropTypes.shape({})).isRequired,
        /**
         * Measurement Store
         */
        measurementStore: PropTypes.instanceOf(MeasurementStore),
        /**
         * Layout system to use for items
         */
        layout: PropTypes.oneOfType([
            PropTypes.instanceOf(LegacyMasonryLayout),
            PropTypes.instanceOf(LegacyUniformRowLayout),
            PropTypes.symbol,
        ]),
        /**
         * A callback which the grid calls when we need to load more items as the user scrolls.
         * The callback should update the state of the items, and pass those in as props
         * to this component.
         */
        loadItems: PropTypes.func,
        /**
         * Minimum number of columns to display.
         */
        minCols: PropTypes.number,
        /**
         * Function that the grid calls to get the scroll container.
         * This is required if the grid is expected to be scrollable.
         */
        scrollContainer: PropTypes.func,
        /**
         * Whether or not to use actual virtualization
         */
        virtualize: PropTypes.bool,
        /**
         * className
         */
        className: PropTypes.string,
        /**
         * itemClassName
         */
        itemClassName: PropTypes.string,
    };
    Masonry.defaultProps = {
        columnWidth: 236,
        measurementStore: new MeasurementStore(),
        minCols: 3,
        layout: DefaultLayoutSymbol,
        loadItems: function () { },
        virtualize: false,
        className: "Masonry",
        itemClassName: "Masonry__Item Masonry__Item__Mounted",
    };
    return Masonry;
}(React.Component));
export default Masonry;
//# sourceMappingURL=Masonry.js.map