/**
 * ScrollContainer is a pass-through component that simply sets up an onScroll
 * handler on the given scrollContainer element (or the element that is
 * returned as result of calling the scrollContainer method). This allows for
 * the event listener subscription of the scrollContainer to be managed inside
 * the React lifecycle without adding bloat to Masonry or other onScroll
 * subscribers.
 *
 * Note that this Component renders its children without creating any
 * additional content. Also note that, while the component is built to manage
 * onScroll inside of the React lifecycle, it doesn't change onScroll events
 * or the API at all, so it could easily be adapted to other event types.
 */
import * as React from 'react';
import PropTypes from 'prop-types';
interface Props {
    children?: React.ReactNode;
    onScroll: (event: Event) => void;
    scrollContainer?: HTMLElement | (() => HTMLElement | null);
}
export default class ScrollContainer extends React.Component<Props> {
    static propTypes: {
        children: PropTypes.Validator<string | number | boolean | {} | React.ReactElement<any> | React.ReactNodeArray | React.ReactPortal>;
        onScroll: PropTypes.Validator<(...args: any[]) => any>;
        scrollContainer: PropTypes.Validator<object>;
    };
    componentDidMount(): void;
    componentDidUpdate(): void;
    componentWillUnmount(): void;
    getScrollContainerRef: () => HTMLElement | undefined;
    handleScroll: (event: Event) => void;
    updateScrollContainer(scrollContainer: HTMLElement): void;
    scrollContainer?: HTMLElement;
    render(): React.ReactElement<any>;
}
export {};
