var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * ScrollContainer is a pass-through component that simply sets up an onScroll
 * handler on the given scrollContainer element (or the element that is
 * returned as result of calling the scrollContainer method). This allows for
 * the event listener subscription of the scrollContainer to be managed inside
 * the React lifecycle without adding bloat to Masonry or other onScroll
 * subscribers.
 *
 * Note that this Component renders its children without creating any
 * additional content. Also note that, while the component is built to manage
 * onScroll inside of the React lifecycle, it doesn't change onScroll events
 * or the API at all, so it could easily be adapted to other event types.
 */
import * as React from 'react';
import PropTypes from 'prop-types';
function getScrollContainer(scrollContainer) {
    return typeof scrollContainer === 'function'
        ? scrollContainer()
        : scrollContainer;
}
var ScrollContainer = /** @class */ (function (_super) {
    __extends(ScrollContainer, _super);
    function ScrollContainer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.getScrollContainerRef = function () { return _this.scrollContainer; };
        _this.handleScroll = function (event) {
            _this.props.onScroll(event);
        };
        return _this;
    }
    ScrollContainer.prototype.componentDidMount = function () {
        var scrollContainer = getScrollContainer(this.props.scrollContainer);
        if (scrollContainer) {
            this.updateScrollContainer(scrollContainer);
        }
    };
    ScrollContainer.prototype.componentDidUpdate = function () {
        var nextScrollContainer = getScrollContainer(this.props.scrollContainer);
        if (nextScrollContainer && nextScrollContainer !== this.scrollContainer) {
            this.updateScrollContainer(nextScrollContainer);
        }
    };
    ScrollContainer.prototype.componentWillUnmount = function () {
        if (this.scrollContainer) {
            this.scrollContainer.removeEventListener('scroll', this.handleScroll);
        }
    };
    ScrollContainer.prototype.updateScrollContainer = function (scrollContainer) {
        if (this.scrollContainer) {
            // cleanup existing scroll container if it exists
            this.scrollContainer.removeEventListener('scroll', this.handleScroll);
        }
        this.scrollContainer = scrollContainer;
        this.scrollContainer.addEventListener('scroll', this.handleScroll);
    };
    ScrollContainer.prototype.render = function () {
        return React.Children.only(this.props.children);
    };
    ScrollContainer.propTypes = {
        children: PropTypes.node.isRequired,
        onScroll: PropTypes.func.isRequired,
        scrollContainer: PropTypes.oneOfType([PropTypes.object, PropTypes.func])
            .isRequired,
    };
    return ScrollContainer;
}(React.Component));
export default ScrollContainer;
//# sourceMappingURL=ScrollContainer.js.map