/**
 * throttle limits the number of times a function can be called to a
 * given threshhold (100ms by default). The function is always called
 * on the leading and trailing edge.
 */
// @flow
export default function throttle(fn, threshhold) {
    if (threshhold === void 0) { threshhold = 100; }
    var last;
    var deferTimer;
    var throttled = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var now = Date.now();
        if (last !== undefined && now - last < threshhold) {
            clearTimeout(deferTimer);
            deferTimer = setTimeout(function () {
                last = now;
                fn.apply(void 0, args);
            }, threshhold - (now - last));
        }
        else {
            last = now;
            fn.apply(void 0, args);
        }
    };
    throttled.clearTimeout = function () {
        if (deferTimer) {
            clearTimeout(deferTimer);
        }
    };
    return throttled;
}
//# sourceMappingURL=throttle.js.map